@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 3978 2008-06-16 16:30:59Z wwalker $"
-__version__   = "$Revision: 3978 $"
-__date__      = "$Date: 2008-06-16 12:30:59 -0400 (Mon, 16 Jun 2008) $"
+__id__        = "$Id: gnomespeechfactory.py 4433 2009-01-19 15:46:14Z wwalker $"
+__version__   = "$Revision: 4433 $"
+__date__      = "$Date: 2009-01-19 10:46:14 -0500 (Mon, 19 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -894,8 +894,6 @@ class SpeechServer(speechserver.SpeechServer):
             return -1
 
         text = self.__addVerbalizedPunctuation(text)
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
-            text = orca_state.activeScript.adjustForPronunciation(text)
 
         try:
             # [[[TODO: WDW - back this stop out for now.  The problem is
@@ -1131,7 +1129,7 @@ class SpeechServer(speechserver.SpeechServer):
             debug.println(debug.LEVEL_SEVERE,
                           "Something looks wrong with speech.  Aborting.")
             debug.printStack(debug.LEVEL_ALL)
-            orca.abort(50)
+            orca.die(50)
         else:
             self.__lastResetTime = time.time()
 
