@@ -27,9 +27,9 @@ The following script specific key sequences are supported:
   Insert-[1-9]  -  Speak and braille a previous chat room message.
 """
 
-__id__        = "$Id: script.py 3932 2008-05-28 17:06:39Z richb $"
-__version__   = "$Revision: 3932 $"
-__date__      = "$Date: 2008-05-28 13:06:39 -0400 (Wed, 28 May 2008) $"
+__id__        = "$Id: script.py 4028 2008-07-11 18:27:51Z joanied $"
+__version__   = "$Revision: 4028 $"
+__date__      = "$Date: 2008-07-11 14:27:51 -0400 (Fri, 11 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -224,22 +224,22 @@ class Script(default.Script):
         keyBindings.add(
             keybindings.KeyBinding(
                 "h",
-                1 << settings.MODIFIER_ORCA,
-                1 << settings.MODIFIER_ORCA,
+                settings.defaultModifierMask,
+                settings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["togglePrefixHandler"]))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "",
-                None,
-                None,
+                settings.defaultModifierMask,
+                settings.NO_MODIFIER_MASK,
                 self.inputEventHandlers["toggleBuddyTypingHandler"]))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "",
-                None,
-                None,
+                settings.defaultModifierMask,
+                settings.NO_MODIFIER_MASK,
                 self.inputEventHandlers["toggleMessageHistoriesHandler"]))
 
         # keybindings to provide chat room message history.
@@ -249,8 +249,8 @@ class Script(default.Script):
             keyBindings.add(
                 keybindings.KeyBinding(
                     messageKey,
-                    1 << settings.MODIFIER_ORCA,
-                    1 << settings.MODIFIER_ORCA,
+                    settings.defaultModifierMask,
+                    settings.ORCA_MODIFIER_MASK,
                     self.inputEventHandlers["reviewMessage"]))
 
         return keyBindings
@@ -756,12 +756,15 @@ class Script(default.Script):
             # If the user doesn't want announcements for when their buddies
             # are typing (or have stopped typing), and this is such a message,
             # then just return. The only reliable way to identify such text
-            # is by the scale.  We also want to store the last message because
-            # msn seems to be sending a constant stream of "is typing" updates.
+            # is by the scale.  This attribute seems to have been removed by
+            # pidgin, so we'll also check the weight.  We also want to store
+            # the last message because msn seems to be sending a constant
+            # stream of "is typing" updates.
             #
             attr, start, end = \
                 self.getTextAttributes(event.source, event.detail1)
-            if float(attr.get('scale', '1')) < 1:
+            if float(attr.get('scale', '1')) < 1 \
+               or int(attr.get('weight', '400')) < 400:
                 if not script_settings.announceBuddyTyping or \
                        self.lastStatus == message:
                     return
