@@ -23,12 +23,13 @@ implementation in Gecko:
 http://developer.mozilla.org/en/docs/Accessibility/ATSPI_Support
 """
 
-__id__        = "$Id: speech_generator.py 4535 2009-02-13 00:55:11Z wwalker $"
-__version__   = "$Revision: 4535 $"
-__date__      = "$Date: 2009-02-12 19:55:11 -0500 (Thu, 12 Feb 2009) $"
+__id__        = "$Id: speech_generator.py 4399 2009-01-02 21:02:45Z joanied $"
+__version__   = "$Revision: 4399 $"
+__date__      = "$Date: 2009-01-02 16:02:45 -0500 (Fri, 02 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import math
 import pyatspi
 
 import orca.rolenames as rolenames
@@ -90,27 +91,6 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         return utterances
 
-    def _getSpeechForAlert(self, obj, already_focused):
-        """Gets the speech for an alert.  What we do here is first try
-        to see if the alert contains text via embedded object characters.
-        If it does, we speak that text.  If it doesn't, we defer to the
-        super class.  The prototype alert we're shooting for is the one
-        from http://bugzilla.gnome.org/show_bug.cgi?id=570551
-
-        Arguments:
-        - obj: an Accessible
-        - already_focused: False if object just received focus
-
-        Returns a list of utterances to be spoken for the object.
-        """
-
-        text = self._script.expandEOCs(obj)
-        if text:
-            return [text]
-        else:
-            return speechgenerator.SpeechGenerator.\
-                       _getSpeechForAlert(self, obj, already_focused)
-
     def _getSpeechForDocumentFrame(self, obj, already_focused):
         """Gets the speech for a document frame.
 
@@ -784,7 +764,40 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
             return speechgenerator.SpeechGenerator.\
                        _getSpeechForSlider(self, obj, already_focused)
 
-        valueString = self._script.getTextForValue(obj)
+        value = obj.queryValue()
+
+        # OK, this craziness is all about trying to figure out the most
+        # meaningful formatting string for the floating point values.
+        # The number of places to the right of the decimal point should
+        # be set by the minimumIncrement, but the minimumIncrement isn't
+        # always set.  So...we'll default the minimumIncrement to 1/100
+        # of the range.  But, if max == min, then we'll just go for showing
+        # them off to two meaningful digits.
+        #
+        try:
+            minimumIncrement = value.minimumIncrement
+        except:
+            minimumIncrement = 0.0
+
+        if minimumIncrement == 0.0:
+            minimumIncrement = (value.maximumValue - value.minimumValue) \
+                               / 100.0
+
+        try:
+            decimalPlaces = max(0, -math.log10(minimumIncrement))
+        except:
+            try:
+                decimalPlaces = max(0, -math.log10(value.minimumValue))
+            except:
+                try:
+                    decimalPlaces = max(0, -math.log10(value.maximumValue))
+                except:
+                    decimalPlaces = 0
+
+        formatter = "%%.%df" % decimalPlaces
+        valueString = formatter % value.currentValue
+        #minString   = formatter % value.minimumValue
+        #maxString   = formatter % value.maximumValue
 
         if already_focused:
             utterances = [valueString]
