@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 4336 2008-11-17 18:15:32Z wwalker $"
-__version__   = "$Revision: 4336 $"
-__date__      = "$Date: 2008-11-17 13:15:32 -0500 (Mon, 17 Nov 2008) $"
+__id__        = "$Id: settings.py 4433 2009-01-19 15:46:14Z wwalker $"
+__version__   = "$Revision: 4433 $"
+__date__      = "$Date: 2009-01-19 10:46:14 -0500 (Mon, 19 Jan 2009) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -105,6 +105,7 @@ userCustomizableSettings = [
     "enableActionKeys",
     "enableNavigationKeys",
     "enableTutorialMessages",
+    "enableMnemonicSpeaking",
     "enableBraille",
     "enableBrailleContext",
     "enableBrailleGrouping",
@@ -161,6 +162,7 @@ userCustomizableSettings = [
     "sayAllStyle",
     "keyboardLayout",
     "speakBlankLines",
+    "speakMultiCaseStringsAsWords",
     "enabledSpokenTextAttributes",
     "enabledBrailledTextAttributes",
     "textAttributesBrailleIndicator",
@@ -574,9 +576,14 @@ enableActionKeys        = True
 #
 enableNavigationKeys    = False
 
-# If True, roles that have tutorial strings defined will be spoken.
+# If True, tutorial strings defined will be spoken.
+#
 enableTutorialMessages = False
 
+# If True, mnemonics will be spoken.
+#
+enableMnemonicSpeaking = False
+
 # If True, show the main Orca window.
 #
 showMainWindow          = True
@@ -623,6 +630,10 @@ outlineStyle = OUTLINE_BOX
 #
 speakBlankLines         = True
 
+# if True, process multi case strings as words.
+#
+speakMultiCaseStringsAsWords = False
+
 # If True, reads all the table cells in the current row rather than just
 # the current one.
 #
@@ -1000,6 +1011,15 @@ setScriptMapping(re.compile(_('Bon Echo')), "Mozilla")
 #
 setScriptMapping(re.compile(_('Minefield')), "Mozilla")
 
+# Translators: see the regular expression note above.  This is for a
+# version of Mozilla Firefox, which chooses to create strange names
+# for itself at the drop of a hat. [[[TODO - JD: Not marked for
+# translation due to string freeze. I'm not convinced it needs to
+# be translated either.]]]
+#
+#setScriptMapping(re.compile(_('Shiretoko')), "Mozilla")
+setScriptMapping(re.compile('Shiretoko'), "Mozilla")
+
 # This is a temporary fix for the schema/FF 3.0 not being accessible
 # issue. (See GNOME bugs #535827 and #555466.)
 #
