@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,26 +14,29 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py 2528 2007-07-09 14:41:22Z wwalker $"
-__version__   = "$Revision: 2528 $"
-__date__      = "$Date: 2007-07-09 10:41:22 -0400 (Mon, 09 Jul 2007) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: speech.py 4433 2009-01-19 15:46:14Z wwalker $"
+__version__   = "$Revision: 4433 $"
+__date__      = "$Date: 2009-01-19 10:46:14 -0500 (Mon, 19 Jan 2009) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
 log = logging.getLogger("speech")
 
+import re
 import time
 
+import chnames
 import debug
+import keynames
+import orca
 import orca_state
-import platform
 import settings
 
 from acss import ACSS
@@ -43,6 +46,13 @@ from orca_i18n import _           # for gettext support
 #
 _speechserver = None
 
+# regular expressions for multiCaseStrings
+#
+multiCaseReg1 = re.compile("([a-z]+)([A-Z][a-z]+)")
+multiCaseReg2 = re.compile("([a-z]+)([A-Z]+)")
+multiCaseReg3 = re.compile("([A-Z]{2}[A-Z]+)([a-z]+)")
+multiCaseReg4 = re.compile("([A-Z])([A-Z][a-z]+)")
+
 def getSpeechServerFactories():
     """Imports all known SpeechServer factory modules.  Returns a list
     of modules that implement the getSpeechServers method, which
@@ -94,7 +104,7 @@ def init():
                               [''])
     except:
         try:
-            moduleName = moduleName.replace("orca.","",1)
+            moduleName = moduleName.replace("orca.", "", 1)
             factory =  __import__(moduleName,
                                   globals(),
                                   locals(),
@@ -122,6 +132,11 @@ def sayAll(utteranceIterator, progressCallback):
         return
     if _speechserver:
         _speechserver.sayAll(utteranceIterator, progressCallback)
+    else:
+        for [context, acss] in utteranceIterator:
+            logLine = "SPEECH OUTPUT: '" + context.utterance + "'"
+            debug.println(debug.LEVEL_INFO, logLine)
+            log.info(logLine)
 
 def speak(text, acss=None, interrupt=True):
     """Speaks all queued text immediately.  If text is not None,
@@ -147,29 +162,78 @@ def speak(text, acss=None, interrupt=True):
     if settings.silenceSpeech:
         return
 
-    debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + text + "'")
-    log.info("'%s'" % text)
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+    if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        text = orca_state.activeScript.adjustForPronunciation(text)
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+
+    logLine = "SPEECH OUTPUT: '" + text + "'"
+    debug.println(debug.LEVEL_INFO, logLine)
+    log.info(logLine)
 
     if _speechserver:
         _speechserver.speak(text, __resolveACSS(acss), interrupt)
 
-def speakKeyEvent(event_string, type):
+def speakKeyEvent(event_string, eventType):
     """Speaks a key event immediately.
 
     Arguments:
     - event_string: string representing the key event as defined by
                     input_event.KeyboardEvent.
-    - type:         key event type as one of orca.KeyEventType constants.
+    - eventType:    key event type as one of orca.KeyEventType constants.
 
     """
     if settings.silenceSpeech:
         return
 
     if _speechserver:
-        _speechserver.speakKeyEvent(event_string, type)
+        _speechserver.speakKeyEvent(event_string, eventType)
+    else:
+        # Check to see if there are localized words to be spoken for
+        # this key event.
+        #
+        event_string = keynames.getKeyName(event_string)
+
+        if eventType == orca.KeyEventType.LOCKING_LOCKED:
+            # Translators: this represents the state of a locking modifier
+            # key (e.g., Caps Lock)
+            #
+            event_string += " " + _("on")
+        elif eventType == orca.KeyEventType.LOCKING_UNLOCKED:
+            # Translators: this represents the state of a locking modifier
+            # key (e.g., Caps Lock)
+            #
+            event_string += " " + _("off")
+
+        logLine = "SPEECH OUTPUT: '" + event_string +"'"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
+
+def speakCharacter(character, acss=None):
+    """Speaks a single character immediately.
+
+    Arguments:
+    - character: text to be spoken
+    - acss:      acss.ACSS instance; if None,
+                 the default voice settings will be used.
+                 Otherwise, the acss settings will be
+                 used to augment/override the default
+                 voice settings.
+    """
+    if settings.silenceSpeech:
+        return
+
+    spokenCharacter = chnames.getCharacterName(character)
+    debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + spokenCharacter + "'")
+    log.info("SPEECH OUTPUT: '%s'" % spokenCharacter)
+
+    if _speechserver:
+        _speechserver.speakCharacter(character, acss=acss)
 
 def isSpeaking():
-    """"Returns True if the system is currently speaking."""
+    """Returns True if the system is currently speaking."""
     if _speechserver:
         return _speechserver.isSpeaking()
     else:
@@ -196,17 +260,35 @@ def speakUtterances(utterances, acss=None, interrupt=True):
 
     if settings.silenceSpeech:
         return
-
-    for utterance in utterances:
-        debug.println(debug.LEVEL_INFO,
-                      "SPEECH OUTPUT: '" + utterance + "'")
-        log.info("'%s'" % utterance)
+    i = 0
+    length = len(utterances)
+    while ( i < length ):
+        utterance = utterances[i]
+        if settings.speakMultiCaseStringsAsWords:
+            utterance = _processMultiCaseString(utterance)
+        if settings.speakMultiCaseStringsAsWords:
+            utterances[i] = _processMultiCaseString(utterance)
+            utterance = utterances[i] 
+        if settings.speakMultiCaseStringsAsWords:
+            utterance = _processMultiCaseString(utterance)
+        i = i + 1
+        
+        logLine = "SPEECH OUTPUT: '" + utterance + "'"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
 
     if _speechserver:
         _speechserver.speakUtterances(utterances,
                                        __resolveACSS(acss),
                                        interrupt)
 
+def getInfo():
+    info = None
+    if _speechserver:
+        info = _speechserver.getInfo()
+
+    return info
+
 def stop():
     if _speechserver:
         _speechserver.stop()
@@ -214,21 +296,41 @@ def stop():
 def increaseSpeechRate(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechRate()
+    else:
+        logLine = "SPEECH OUTPUT: 'faster'"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
+
     return True
 
 def decreaseSpeechRate(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.decreaseSpeechRate()
+    else:
+        logLine = "SPEECH OUTPUT: 'slower'"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
+
     return True
 
 def increaseSpeechPitch(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechPitch()
+    else:
+        logLine = "SPEECH OUTPUT: 'higher'"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
+
     return True
 
 def decreaseSpeechPitch(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.decreaseSpeechPitch()
+    else:
+        logLine = "SPEECH OUTPUT: 'lower'"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
+
     return True
 
 def shutdown():
@@ -238,7 +340,6 @@ def shutdown():
         _speechserver = None
 
 def reset(text=None, acss=None):
-    global _speechserver
     if _speechserver:
         _speechserver.reset(text, acss)
 
@@ -270,4 +371,14 @@ def test():
                 server.shutdown()
             except:
                 debug.printException(debug.LEVEL_OFF)
-                pass
+
+def _processMultiCaseString(string):
+    """Helper function, applies the regexes to split multiCaseStrings
+    to multiple words.
+    """
+
+    string = multiCaseReg1.sub('\\1 \\2', string)
+    string = multiCaseReg2.sub('\\1 \\2', string)
+    string = multiCaseReg3.sub('\\1 \\2', string)    
+    string = multiCaseReg4.sub('\\1 \\2', string)
+    return string
