@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id: speechgenerator.py 4426 2009-01-17 02:21:34Z wwalker $"
+__version__   = "$Revision: 4426 $"
+__date__      = "$Date: 2009-01-16 21:21:34 -0500 (Fri, 16 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -78,6 +78,8 @@ class SpeechGenerator:
              self._getSpeechForDialog
         self.speechGenerators[pyatspi.ROLE_DIRECTORY_PANE]      = \
              self._getSpeechForDirectoryPane
+        self.speechGenerators[pyatspi.ROLE_EMBEDDED]            = \
+             self._getSpeechForEmbedded
         self.speechGenerators[pyatspi.ROLE_FRAME]               = \
              self._getSpeechForFrame
         self.speechGenerators[pyatspi.ROLE_HTML_CONTAINER]      = \
@@ -92,6 +94,8 @@ class SpeechGenerator:
              self._getSpeechForLayeredPane
         self.speechGenerators[pyatspi.ROLE_LIST]                = \
              self._getSpeechForList
+        self.speechGenerators[pyatspi.ROLE_LIST_ITEM]           = \
+             self._getSpeechForListItem
         self.speechGenerators[pyatspi.ROLE_MENU]                = \
              self._getSpeechForMenu
         self.speechGenerators[pyatspi.ROLE_MENU_BAR]            = \
@@ -151,25 +155,53 @@ class SpeechGenerator:
         self.speechGenerators[pyatspi.ROLE_WINDOW]              = \
              self._getSpeechForWindow
 
-    def _getSpeechForObjectAccelerator(self, obj):
-        """Returns a list of utterances that describes the keyboard
-        accelerator (and possibly shortcut) for the given object.
+    def _addSpeechForObjectAccelerator(self, obj, utterances):
+        """Adds an utterance that describes the keyboard accelerator for the
+        given object to the list of utterances passed in.
 
         Arguments:
         - obj: the Accessible object
+        - utterances: the list of utterances to add to.
 
         Returns a list of utterances to be spoken.
         """
 
-        utterances = []
+        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            [mnemonic, shortcut, accelerator] = self._script.getKeyBinding(obj)
+            if accelerator:
+                # Add punctuation for better prosody.
+                #
+                #if utterances:
+                #    utterances[-1] += "."
+                utterances.append(accelerator)
 
-        result = self._script.getKeyBinding(obj)
-        accelerator = result[2]
+    def _addSpeechForObjectMnemonic(self, obj, utterances):
+        """Adds an utterance that describes the mnemonic for the given object
+        to the utterances passed in.
 
-        if len(accelerator) > 0:
-            utterances.append(accelerator)
+        Arguments:
+        - obj: the Accessible object
+        - utterances: the list of utterances to add to.
+        """
 
-        return utterances
+        if settings.enableMnemonicSpeaking:
+            # The mnemonic is what we're really looking for.  But,
+            # some objects (e.g., menu items) only expose the mnemonic
+            # via the full shortcut (e.g., "Alt f c" for a "Close"
+            # menu item).  So, we fall back to the last character in
+            # the shortcut if the shortcut exists.
+            #
+            [mnemonic, shortcut, accelerator] = self._script.getKeyBinding(obj)
+            if mnemonic:
+                mnemonic = mnemonic[-1] # we just want a single character
+            if not mnemonic and shortcut:
+                mnemonic = shortcut
+            if mnemonic:
+                # Add punctuation for better prosody.
+                #
+                #if utterances:
+                #    utterances[-1] += "."
+                utterances.append(mnemonic)
 
     def _getSpeechForObjectAvailability(self, obj):
         """Returns a list of utterances that describes the availability
@@ -205,7 +237,7 @@ class SpeechGenerator:
         else:
             return []
 
-    def _getSpeechForObjectRole(self, obj, role=None):
+    def getSpeechForObjectRole(self, obj, role=None):
         if (obj.getRole() != pyatspi.ROLE_UNKNOWN):
             return [rolenames.getSpeechForRoleName(obj, role)]
         else:
@@ -239,6 +271,25 @@ class SpeechGenerator:
 
         return utterance
 
+    def _getSpeechForRequiredObject(self, obj):
+        """Returns the list of utterances that describe the required state
+        of the given object.
+
+        Arguments:
+        - obj: the Accessible object
+
+        Returns a list of utterances to be spoken.
+        """
+
+        if not settings.presentRequiredState:
+            return []
+
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_REQUIRED):
+            return [settings.speechRequiredStateString]
+        else:
+            return []
+
     def _debugGenerator(self, generatorName, obj, already_focused, utterances):
         """Prints debug.LEVEL_FINER information regarding the speech generator.
 
@@ -271,7 +322,7 @@ class SpeechGenerator:
 
         The default speech will be of the following form:
 
-        label name role availability
+        label name role availability mnemonic
 
         Arguments:
         - obj: an Accessible
@@ -291,9 +342,10 @@ class SpeechGenerator:
             if name != label:
                 utterances.extend(name)
             utterances.extend(self._getSpeechForAllTextSelection(obj))
-            utterances.extend(self._getSpeechForObjectRole(obj, role))
-
-        utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj, role))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            if obj == orca_state.locusOfFocus:
+                self._addSpeechForObjectMnemonic(obj, utterances)
 
         self._debugGenerator("_getDefaultSpeech",
                              obj,
@@ -393,6 +445,7 @@ class SpeechGenerator:
         """
 
         utterances = []
+
         state = obj.getState()
         if state.contains(pyatspi.STATE_INDETERMINATE):
             # Translators: this represents the state of a checkbox.
@@ -417,12 +470,14 @@ class SpeechGenerator:
                 utterances.extend(name)
             if obj.getRole() == pyatspi.ROLE_TABLE_CELL:
                 utterances.extend(
-                  self._getSpeechForObjectRole(
+                  self.getSpeechForObjectRole(
                     obj, pyatspi.ROLE_CHECK_BOX))
             else:
-                utterances.extend(self._getSpeechForObjectRole(obj))
+                utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(checkedState)
+            utterances.extend(self._getSpeechForRequiredObject(obj))
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         else:
             utterances.append(checkedState)
 
@@ -446,9 +501,8 @@ class SpeechGenerator:
 
         utterances = self._getSpeechForCheckBox(obj, already_focused)
 
-        if (settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE)\
-           and not already_focused:
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        if not already_focused:
+            self._addSpeechForObjectAccelerator(obj, utterances)
 
         self._debugGenerator("_getSpeechForCheckMenuItem",
                              obj,
@@ -500,14 +554,15 @@ class SpeechGenerator:
             utterances.extend(name)
 
         if not already_focused:
-            utterances.extend(self._getSpeechForObjectRole(obj))
-
-        utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
 
         for child in obj:
             if child.getRole() == pyatspi.ROLE_TEXT:
                 utterances.extend(self._getSpeechForAllTextSelection(child))
 
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+        self._addSpeechForObjectMnemonic(obj, utterances)
+
         self._debugGenerator("_getSpeechForComboBox",
                              obj,
                              already_focused,
@@ -594,6 +649,39 @@ class SpeechGenerator:
 
         return utterances
 
+    def _getSpeechForEmbedded(self, obj, already_focused, role=None):
+        """Gets a list of utterances to be spoken for the current
+        embedded component (i.e., something in a panel).
+
+        Arguments:
+        - obj: an Accessible
+        - already_focused: False if object just received focus
+        - role: A role that should be used instead of the Accessible's 
+          possible role.
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+
+        label = self._getSpeechForObjectLabel(obj)
+        utterances.extend(label)
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
+        if not utterances:
+            try:
+                utterances.append(obj.getApplication().name)
+            except:
+                pass
+
+        self._debugGenerator("_getSpeechForEmbedded",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        return utterances
+
     def _getSpeechForFrame(self, obj, already_focused):
         """Get the speech for a frame.
 
@@ -604,13 +692,33 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        # [[[TODO: richb - readjusted to just get the default speech instead
-        # of treating the frame like an alert and speaking all unrelated
-        # labels. We'll need to see if this has any adverse effects and
-        # adjust accordingly.]]]
-        #
-        utterances = self._getDefaultSpeech(obj, already_focused)
-        #utterances = self._getSpeechForAlert(obj, already_focused)
+        utterances = []
+
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
+
+            # If this application has more than one unfocused alert or
+            # dialog window, then speak '<m> unfocused dialogs'
+            # to let the user know.
+            #
+            alertAndDialogCount = \
+                        self._script.getUnfocusedAlertAndDialogCount(obj)
+            if alertAndDialogCount > 0:
+                # Translators: this tells the user how many unfocused
+                # alert and dialog windows that this application has.
+                #
+                line = ngettext("%d unfocused dialog",
+                                "%d unfocused dialogs",
+                                alertAndDialogCount) % alertAndDialogCount
+                utterances.append(line)
+
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
 
         self._debugGenerator("_getSpeechForFrame",
                              obj,
@@ -786,7 +894,15 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getDefaultSpeech(obj, already_focused)
+        utterances = []
+
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
 
         # If already in focus then the tree probably collapsed or expanded
         state = obj.getState()
@@ -804,6 +920,8 @@ class SpeechGenerator:
                 #
                 utterances.append(_("collapsed"))
                 
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+
         self._debugGenerator("_getSpeechForListItem",
                              obj,
                              already_focused,
@@ -820,12 +938,19 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getDefaultSpeech(obj, already_focused)
+        utterances = []
 
-        if (obj == orca_state.locusOfFocus) \
-               and (settings.speechVerbosityLevel \
-                    == settings.VERBOSITY_LEVEL_VERBOSE):
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
+            self._addSpeechForObjectAccelerator(obj, utterances)
 
         self._debugGenerator("_getSpeechForMenu",
                              obj,
@@ -845,7 +970,6 @@ class SpeechGenerator:
         """
 
         utterances = self._getDefaultSpeech(obj, already_focused)
-
         self._debugGenerator("_getSpeechForMenuBar",
                              obj,
                              already_focused,
@@ -863,12 +987,11 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
+        utterances = []
+
         # No need to say "menu item" because we already know that.
         #
-        utterances = self._getSpeechForObjectName(obj)
-        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances.extend(self._getSpeechForObjectAvailability(obj))
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        utterances.extend(self._getSpeechForObjectName(obj))
 
         # OpenOffice check menu items currently have a role of "menu item"
         # rather then "check menu item", so we need to test if one of the
@@ -883,6 +1006,11 @@ class SpeechGenerator:
             #
             utterances.append(_("checked"))
 
+        if not already_focused:
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
+            self._addSpeechForObjectAccelerator(obj, utterances)
+
         self._debugGenerator("_getSpeechForMenuItem",
                              obj,
                              already_focused,
@@ -909,14 +1037,21 @@ class SpeechGenerator:
         if len(utterances) == 0:
             if obj.name and (len(obj.name)):
                 utterances.append(obj.name)
+
+        if settings.presentReadOnlyText \
+           and self._script.isReadOnlyTextArea(obj):
+            utterances.append(settings.speechReadOnlyString)
+
         if obj.getRole() != pyatspi.ROLE_PARAGRAPH:
-            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
 
         [text, caretOffset, startOffset] = self._script.getTextLineAtCaret(obj)
         utterances.append(text)
 
         utterances.extend(self._getSpeechForAllTextSelection(obj))
 
+        self._addSpeechForObjectMnemonic(obj, utterances)
+
         self._debugGenerator("_getSpeechForText",
                              obj,
                              already_focused,
@@ -1013,7 +1148,7 @@ class SpeechGenerator:
             name = self._getSpeechForObjectName(obj)
             if name != label:
                 utterances.extend(name)
-            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
 
         utterances.append(percentage)
 
@@ -1055,6 +1190,7 @@ class SpeechGenerator:
         """
 
         utterances = []
+
         state = obj.getState()
         if state.contains(pyatspi.STATE_CHECKED):
             # Translators: this is in reference to a radio button being
@@ -1079,8 +1215,9 @@ class SpeechGenerator:
             #utterances.extend(self._getSpeechForObjectLabel(obj))
             utterances.extend(self._getSpeechForObjectName(obj))
             utterances.append(selectionState)
-            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         else:
             utterances.append(selectionState)
 
@@ -1096,17 +1233,17 @@ class SpeechGenerator:
         already had focus, then only the state is spoken.
 
         Arguments:
-        - obj: the check menu item
+        - obj: the radio menu item
         - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getSpeechForRadioButton(obj, False)
+        utterances = []
+        utterances.extend(self._getSpeechForRadioButton(obj, False))
 
-        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances.extend(self._getSpeechForObjectAvailability(obj))
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        if not already_focused:
+            self._addSpeechForObjectAccelerator(obj, utterances)
 
         self._debugGenerator("_getSpeechForRadioMenuItem",
                              obj,
@@ -1187,10 +1324,11 @@ class SpeechGenerator:
             #
             if not utterances:
                 utterances.extend(self._getSpeechForObjectName(obj))
-            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(valueString)
+            utterances.extend(self._getSpeechForRequiredObject(obj))
             utterances.extend(self._getSpeechForObjectAvailability(obj))
-
+            self._addSpeechForObjectMnemonic(obj, utterances)
         self._debugGenerator("_getSpeechForSlider",
                              obj,
                              already_focused,
@@ -1213,6 +1351,7 @@ class SpeechGenerator:
             utterances = [self._script.getDisplayedText(obj)]
         else:
             utterances = self._getDefaultSpeech(obj, already_focused)
+            utterances.extend(self._getSpeechForRequiredObject(obj))
 
         self._debugGenerator("_getSpeechForSpinButton",
                              obj,
@@ -1240,9 +1379,10 @@ class SpeechGenerator:
             utterances.extend(self._getSpeechForObjectLabel(obj))
             if not utterances:
                 utterances.extend(self._getSpeechForObjectName(obj))
-            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(valueString)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
 
         self._debugGenerator("_getSpeechForSplitPane",
                              obj,
@@ -1408,6 +1548,8 @@ class SpeechGenerator:
                 #
                 utterances.append(_("collapsed"))
 
+        utterances.extend(self._getSpeechForRequiredObject(obj))
+
         self._debugGenerator("_getSpeechForTableCell",
                              obj,
                              already_focused,
@@ -1436,8 +1578,9 @@ class SpeechGenerator:
             if settings.readTableCellRow and parent_table \
                 and (not self._script.isLayoutOnly(obj.parent)):
                 parent = obj.parent
-                row = parent_table.getRowAtIndex(obj.getIndexInParent())
-                column = parent_table.getColumnAtIndex(obj.getIndexInParent())
+                index = self._script.getCellIndex(obj)
+                row = parent_table.getRowAtIndex(index)
+                column = parent_table.getColumnAtIndex(index)
 
                 # This is an indication of whether we should speak all the
                 # table cells (the user has moved focus up or down a row),
@@ -1445,8 +1588,8 @@ class SpeechGenerator:
                 # the same row).
                 #
                 speakAll = True
-                if self._script.pointOfReference.has_key("lastRow") and \
-                    self._script.pointOfReference.has_key("lastColumn"):
+                if "lastRow" in self._script.pointOfReference and \
+                    "lastColumn" in self._script.pointOfReference:
                     pointOfReference = self._script.pointOfReference
                     speakAll = (pointOfReference["lastRow"] != row) or \
                         ((row == 0 or row == parent_table.nRows-1) and \
@@ -1605,8 +1748,10 @@ class SpeechGenerator:
         """
 
         utterances = []
+
         state = obj.getState()
-        if state.contains(pyatspi.STATE_CHECKED):
+        if state.contains(pyatspi.STATE_CHECKED) \
+           or state.contains(pyatspi.STATE_PRESSED):
             # Translators: the state of a toggle button.
             #
             checkedState = _("pressed")
@@ -1623,9 +1768,10 @@ class SpeechGenerator:
             name = self._getSpeechForObjectName(obj)
             if name != label:
                 utterances.extend(name)
-            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(checkedState)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         else:
             utterances.append(checkedState)
 
@@ -1725,7 +1871,7 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken.
         """
         role = obj.getRole()
-        if self.speechGenerators.has_key(role):
+        if role in self.speechGenerators:
             generator = self.speechGenerators[role]
         else:
             generator = self._getDefaultSpeech
@@ -1771,12 +1917,11 @@ class SpeechGenerator:
                     #
                     if parent.getRole() not in [pyatspi.ROLE_TABLE_CELL,
                                                 pyatspi.ROLE_FILLER]:
-                        utterances.append(\
-                            rolenames.getSpeechForRoleName(parent))
+                        utterances.extend(self.getSpeechForObjectRole(parent))
                     utterances.append(text)
                     if parent.getRole() == pyatspi.ROLE_TABLE_CELL:
-                        utterances.append(\
-                            rolenames.getSpeechForRoleName(parent))
+                        utterances.extend(self.getSpeechForObjectRole(parent))
+
             parent = parent.parent
 
         utterances.reverse()
